@@ -25,7 +25,7 @@
 
 toolstartmessage <- function(level=NULL) {
   functioncall <- paste(deparse(sys.call(-1)),collapse="")
-  vcat(1,"Run",functioncall,level=level)
+  vcat(-2,"Run",functioncall, level=level, fill=300)
   startdata <- list(time1=proc.time())
   d <- getConfig("diagnostics")
   if(is.character(d)) {
